"use strict";
var RelationTypes_1 = require("./types/RelationTypes");
/**
 * Contains all entity metadata.
 */
var EntityMetadata = (function () {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    function EntityMetadata(namingStrategy, tableMetadata, columnMetadatas, relationMetadatas, compositeIndexMetadatas) {
        var _this = this;
        this.foreignKeys = [];
        this.namingStrategy = namingStrategy;
        this.table = tableMetadata;
        this.columns = columnMetadatas;
        this.relations = relationMetadatas;
        this.compositeIndices = compositeIndexMetadatas;
        this.table.entityMetadata = this;
        this.relations.forEach(function (relation) { return relation.entityMetadata = _this; });
        this.compositeIndices.forEach(function (index) { return index.entityMetadata = _this; });
    }
    Object.defineProperty(EntityMetadata.prototype, "name", {
        // -------------------------------------------------------------------------
        // Accessors
        // -------------------------------------------------------------------------
        get: function () {
            if (!this.table || !this.table.target)
                throw new Error("No table target set to the entity metadata.");
            return this.table.target.name;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "target", {
        get: function () {
            return this.table.target;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "oneToOneRelations", {
        get: function () {
            return this.relations.filter(function (relation) { return relation.relationType === RelationTypes_1.RelationTypes.ONE_TO_ONE; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "ownerOneToOneRelations", {
        get: function () {
            return this.relations.filter(function (relation) { return relation.relationType === RelationTypes_1.RelationTypes.ONE_TO_ONE && relation.isOwning; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "oneToManyRelations", {
        get: function () {
            return this.relations.filter(function (relation) { return relation.relationType === RelationTypes_1.RelationTypes.ONE_TO_MANY; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "manyToOneRelations", {
        get: function () {
            return this.relations.filter(function (relation) { return relation.relationType === RelationTypes_1.RelationTypes.MANY_TO_ONE; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "manyToManyRelations", {
        get: function () {
            return this.relations.filter(function (relation) { return relation.relationType === RelationTypes_1.RelationTypes.MANY_TO_MANY; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "ownerManyToManyRelations", {
        get: function () {
            return this.relations.filter(function (relation) { return relation.relationType === RelationTypes_1.RelationTypes.MANY_TO_MANY && relation.isOwning; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "relationsWithJoinColumns", {
        get: function () {
            return this.ownerOneToOneRelations.concat(this.manyToOneRelations);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "primaryColumn", {
        get: function () {
            return this.columns.find(function (column) { return column.isPrimary; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "createDateColumn", {
        get: function () {
            return this.columns.find(function (column) { return column.mode === "createDate"; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "updateDateColumn", {
        get: function () {
            return this.columns.find(function (column) { return column.mode === "updateDate"; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "versionColumn", {
        get: function () {
            return this.columns.find(function (column) { return column.mode === "version"; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "treeChildrenCountColumn", {
        get: function () {
            return this.columns.find(function (column) { return column.mode === "treeChildrenCount"; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "treeLevelColumn", {
        get: function () {
            return this.columns.find(function (column) { return column.mode === "treeLevel"; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "hasPrimaryKey", {
        get: function () {
            return !!this.primaryColumn;
        },
        enumerable: true,
        configurable: true
    });
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Creates a new entity.
     */
    EntityMetadata.prototype.create = function () {
        return new this.table.target();
    };
    EntityMetadata.prototype.createPropertiesMap = function () {
        var entity = {};
        this.columns.forEach(function (column) { return entity[column.name] = column.name; });
        this.relations.forEach(function (relation) { return entity[relation.name] = relation.name; });
        return entity;
    };
    EntityMetadata.prototype.getEntityId = function (entity) {
        return entity[this.primaryColumn.propertyName];
    };
    EntityMetadata.prototype.hasColumnWithPropertyName = function (propertyName) {
        return !!this.findColumnWithPropertyName(propertyName);
    };
    EntityMetadata.prototype.hasColumnWithDbName = function (name) {
        return !!this.findColumnWithDbName(name);
    };
    EntityMetadata.prototype.findColumnWithPropertyName = function (propertyName) {
        return this.columns.find(function (column) { return column.propertyName === propertyName; });
    };
    EntityMetadata.prototype.findColumnWithDbName = function (name) {
        return this.columns.find(function (column) { return column.name === name; });
    };
    EntityMetadata.prototype.hasRelationWithPropertyName = function (propertyName) {
        return !!this.findRelationWithPropertyName(propertyName);
    };
    EntityMetadata.prototype.hasRelationWithDbName = function (dbName) {
        return !!this.findRelationWithDbName(dbName);
    };
    EntityMetadata.prototype.findRelationWithPropertyName = function (propertyName) {
        return this.relations.find(function (relation) { return relation.propertyName === propertyName; });
    };
    EntityMetadata.prototype.findRelationWithDbName = function (propertyName) {
        return this.relations.find(function (relation) { return relation.name === propertyName; });
    };
    EntityMetadata.prototype.findRelationWithOneWithPropertyName = function (propertyName) {
        return this.relations.find(function (relation) { return relation.propertyName === propertyName && (relation.isOneToMany || relation.isOneToOne); });
    };
    EntityMetadata.prototype.findRelationWithOneWithDbName = function (name) {
        return this.relations.find(function (relation) { return relation.name === name && (relation.isOneToMany || relation.isOneToOne); });
    };
    EntityMetadata.prototype.findRelationWithManyWithPropertyName = function (propertyName) {
        return this.relations.find(function (relation) { return relation.propertyName === propertyName && (relation.isManyToOne || relation.isManyToMany); });
    };
    EntityMetadata.prototype.findRelationWithManyWithDbName = function (name) {
        return this.relations.find(function (relation) { return relation.name === name && (relation.isManyToOne || relation.isManyToMany); });
    };
    EntityMetadata.prototype.hasRelationWithOneWithPropertyName = function (propertyName) {
        return !!this.findRelationWithOneWithPropertyName(propertyName);
    };
    EntityMetadata.prototype.hasRelationWithManyWithPropertyName = function (propertyName) {
        return !!this.findRelationWithManyWithPropertyName(propertyName);
    };
    EntityMetadata.prototype.hasRelationWithOneWithName = function (name) {
        return !!this.findRelationWithOneWithDbName(name);
    };
    EntityMetadata.prototype.hasRelationWithManyWithName = function (name) {
        return !!this.findRelationWithManyWithDbName(name);
    };
    Object.defineProperty(EntityMetadata.prototype, "treeParentRelation", {
        get: function () {
            return this.relations.find(function (relation) { return relation.isTreeParent; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "treeChildrenRelation", {
        get: function () {
            return this.relations.find(function (relation) { return relation.isTreeChildren; });
        },
        enumerable: true,
        configurable: true
    });
    return EntityMetadata;
}());
exports.EntityMetadata = EntityMetadata;
//# sourceMappingURL=EntityMetadata.js.map